# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class SessionCredentials:
    def __init__(self, access_key=None, access_secret=None, security_token=None):
        if access_key is None:
            raise ValueError("accessKey should not be None")
        if access_secret is None:
            raise ValueError("accessSecret should not be None")

        self.access_key = access_key
        self.access_secret = access_secret
        self.security_token = security_token


class SessionCredentialsProvider:
    def __init__(self, credentials):
        if not isinstance(credentials, SessionCredentials):
            raise ValueError("credentials should be an instance of SessionCredentials")
        self.credentials = credentials

    def get_credentials(self):
        return self.credentials



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: apache/rocketmq/v2/definition.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#apache/rocketmq/v2/definition.proto\x12\x12\x61pache.rocketmq.v2\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\"T\n\x10\x46ilterExpression\x12,\n\x04type\x18\x01 \x01(\x0e\x32\x1e.apache.rocketmq.v2.FilterType\x12\x12\n\nexpression\x18\x02 \x01(\t\"\xbb\x01\n\x0bRetryPolicy\x12\x14\n\x0cmax_attempts\x18\x01 \x01(\x05\x12\x45\n\x13\x65xponential_backoff\x18\x02 \x01(\x0b\x32&.apache.rocketmq.v2.ExponentialBackoffH\x00\x12\x43\n\x12\x63ustomized_backoff\x18\x03 \x01(\x0b\x32%.apache.rocketmq.v2.CustomizedBackoffH\x00\x42\n\n\x08strategy\"|\n\x12\x45xponentialBackoff\x12*\n\x07initial\x18\x01 \x01(\x0b\x32\x19.google.protobuf.Duration\x12&\n\x03max\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x12\n\nmultiplier\x18\x03 \x01(\x02\"<\n\x11\x43ustomizedBackoff\x12\'\n\x04next\x18\x01 \x03(\x0b\x32\x19.google.protobuf.Duration\"4\n\x08Resource\x12\x1a\n\x12resource_namespace\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"z\n\x11SubscriptionEntry\x12+\n\x05topic\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12\x38\n\nexpression\x18\x02 \x01(\x0b\x32$.apache.rocketmq.v2.FilterExpression\"%\n\x07\x41\x64\x64ress\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\"n\n\tEndpoints\x12\x31\n\x06scheme\x18\x01 \x01(\x0e\x32!.apache.rocketmq.v2.AddressScheme\x12.\n\taddresses\x18\x02 \x03(\x0b\x32\x1b.apache.rocketmq.v2.Address\"T\n\x06\x42roker\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\x05\x12\x30\n\tendpoints\x18\x03 \x01(\x0b\x32\x1d.apache.rocketmq.v2.Endpoints\"\xe6\x01\n\x0cMessageQueue\x12+\n\x05topic\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12\n\n\x02id\x18\x02 \x01(\x05\x12\x32\n\npermission\x18\x03 \x01(\x0e\x32\x1e.apache.rocketmq.v2.Permission\x12*\n\x06\x62roker\x18\x04 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Broker\x12=\n\x14\x61\x63\x63\x65pt_message_types\x18\x05 \x03(\x0e\x32\x1f.apache.rocketmq.v2.MessageType\"H\n\x06\x44igest\x12,\n\x04type\x18\x01 \x01(\x0e\x32\x1e.apache.rocketmq.v2.DigestType\x12\x10\n\x08\x63hecksum\x18\x02 \x01(\t\"\x8f\x08\n\x10SystemProperties\x12\x10\n\x03tag\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0c\n\x04keys\x18\x02 \x03(\t\x12\x12\n\nmessage_id\x18\x03 \x01(\t\x12/\n\x0b\x62ody_digest\x18\x04 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Digest\x12\x33\n\rbody_encoding\x18\x05 \x01(\x0e\x32\x1c.apache.rocketmq.v2.Encoding\x12\x35\n\x0cmessage_type\x18\x06 \x01(\x0e\x32\x1f.apache.rocketmq.v2.MessageType\x12\x32\n\x0e\x62orn_timestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x11\n\tborn_host\x18\x08 \x01(\t\x12\x38\n\x0fstore_timestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x01\x88\x01\x01\x12\x12\n\nstore_host\x18\n \x01(\t\x12;\n\x12\x64\x65livery_timestamp\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x02\x88\x01\x01\x12\x1b\n\x0ereceipt_handle\x18\x0c \x01(\tH\x03\x88\x01\x01\x12\x10\n\x08queue_id\x18\r \x01(\x05\x12\x19\n\x0cqueue_offset\x18\x0e \x01(\x03H\x04\x88\x01\x01\x12:\n\x12invisible_duration\x18\x0f \x01(\x0b\x32\x19.google.protobuf.DurationH\x05\x88\x01\x01\x12\x1d\n\x10\x64\x65livery_attempt\x18\x10 \x01(\x05H\x06\x88\x01\x01\x12\x1a\n\rmessage_group\x18\x11 \x01(\tH\x07\x88\x01\x01\x12\x1a\n\rtrace_context\x18\x12 \x01(\tH\x08\x88\x01\x01\x12N\n&orphaned_transaction_recovery_duration\x18\x13 \x01(\x0b\x32\x19.google.protobuf.DurationH\t\x88\x01\x01\x12\x43\n\x11\x64\x65\x61\x64_letter_queue\x18\x14 \x01(\x0b\x32#.apache.rocketmq.v2.DeadLetterQueueH\n\x88\x01\x01\x42\x06\n\x04_tagB\x12\n\x10_store_timestampB\x15\n\x13_delivery_timestampB\x11\n\x0f_receipt_handleB\x0f\n\r_queue_offsetB\x15\n\x13_invisible_durationB\x13\n\x11_delivery_attemptB\x10\n\x0e_message_groupB\x10\n\x0e_trace_contextB)\n\'_orphaned_transaction_recovery_durationB\x14\n\x12_dead_letter_queue\"4\n\x0f\x44\x65\x61\x64LetterQueue\x12\r\n\x05topic\x18\x01 \x01(\t\x12\x12\n\nmessage_id\x18\x02 \x01(\t\"\x86\x02\n\x07Message\x12+\n\x05topic\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12H\n\x0fuser_properties\x18\x02 \x03(\x0b\x32/.apache.rocketmq.v2.Message.UserPropertiesEntry\x12?\n\x11system_properties\x18\x03 \x01(\x0b\x32$.apache.rocketmq.v2.SystemProperties\x12\x0c\n\x04\x62ody\x18\x04 \x01(\x0c\x1a\x35\n\x13UserPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"E\n\nAssignment\x12\x37\n\rmessage_queue\x18\x01 \x01(\x0b\x32 .apache.rocketmq.v2.MessageQueue\"A\n\x06Status\x12&\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x18.apache.rocketmq.v2.Code\x12\x0f\n\x07message\x18\x02 \x01(\t\"i\n\x02UA\x12.\n\x08language\x18\x01 \x01(\x0e\x32\x1c.apache.rocketmq.v2.Language\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x10\n\x08platform\x18\x03 \x01(\t\x12\x10\n\x08hostname\x18\x04 \x01(\t\"\x90\x04\n\x08Settings\x12\x38\n\x0b\x63lient_type\x18\x01 \x01(\x0e\x32\x1e.apache.rocketmq.v2.ClientTypeH\x01\x88\x01\x01\x12\x38\n\x0c\x61\x63\x63\x65ss_point\x18\x02 \x01(\x0b\x32\x1d.apache.rocketmq.v2.EndpointsH\x02\x88\x01\x01\x12<\n\x0e\x62\x61\x63koff_policy\x18\x03 \x01(\x0b\x32\x1f.apache.rocketmq.v2.RetryPolicyH\x03\x88\x01\x01\x12\x37\n\x0frequest_timeout\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationH\x04\x88\x01\x01\x12\x34\n\npublishing\x18\x05 \x01(\x0b\x32\x1e.apache.rocketmq.v2.PublishingH\x00\x12\x38\n\x0csubscription\x18\x06 \x01(\x0b\x32 .apache.rocketmq.v2.SubscriptionH\x00\x12*\n\nuser_agent\x18\x07 \x01(\x0b\x32\x16.apache.rocketmq.v2.UA\x12*\n\x06metric\x18\x08 \x01(\x0b\x32\x1a.apache.rocketmq.v2.MetricB\t\n\x07pub_subB\x0e\n\x0c_client_typeB\x0f\n\r_access_pointB\x11\n\x0f_backoff_policyB\x12\n\x10_request_timeout\"p\n\nPublishing\x12,\n\x06topics\x18\x01 \x03(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12\x15\n\rmax_body_size\x18\x02 \x01(\x05\x12\x1d\n\x15validate_message_type\x18\x03 \x01(\x08\"\xb3\x02\n\x0cSubscription\x12\x30\n\x05group\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.ResourceH\x00\x88\x01\x01\x12<\n\rsubscriptions\x18\x02 \x03(\x0b\x32%.apache.rocketmq.v2.SubscriptionEntry\x12\x11\n\x04\x66ifo\x18\x03 \x01(\x08H\x01\x88\x01\x01\x12\x1f\n\x12receive_batch_size\x18\x04 \x01(\x05H\x02\x88\x01\x01\x12<\n\x14long_polling_timeout\x18\x05 \x01(\x0b\x32\x19.google.protobuf.DurationH\x03\x88\x01\x01\x42\x08\n\x06_groupB\x07\n\x05_fifoB\x15\n\x13_receive_batch_sizeB\x17\n\x15_long_polling_timeout\"Y\n\x06Metric\x12\n\n\x02on\x18\x01 \x01(\x08\x12\x35\n\tendpoints\x18\x02 \x01(\x0b\x32\x1d.apache.rocketmq.v2.EndpointsH\x00\x88\x01\x01\x42\x0c\n\n_endpoints*Y\n\x15TransactionResolution\x12&\n\"TRANSACTION_RESOLUTION_UNSPECIFIED\x10\x00\x12\n\n\x06\x43OMMIT\x10\x01\x12\x0c\n\x08ROLLBACK\x10\x02*W\n\x11TransactionSource\x12\x16\n\x12SOURCE_UNSPECIFIED\x10\x00\x12\x11\n\rSOURCE_CLIENT\x10\x01\x12\x17\n\x13SOURCE_SERVER_CHECK\x10\x02*W\n\nPermission\x12\x1a\n\x16PERMISSION_UNSPECIFIED\x10\x00\x12\x08\n\x04NONE\x10\x01\x12\x08\n\x04READ\x10\x02\x12\t\n\x05WRITE\x10\x03\x12\x0e\n\nREAD_WRITE\x10\x04*;\n\nFilterType\x12\x1b\n\x17\x46ILTER_TYPE_UNSPECIFIED\x10\x00\x12\x07\n\x03TAG\x10\x01\x12\x07\n\x03SQL\x10\x02*T\n\rAddressScheme\x12\x1e\n\x1a\x41\x44\x44RESS_SCHEME_UNSPECIFIED\x10\x00\x12\x08\n\x04IPv4\x10\x01\x12\x08\n\x04IPv6\x10\x02\x12\x0f\n\x0b\x44OMAIN_NAME\x10\x03*]\n\x0bMessageType\x12\x1c\n\x18MESSAGE_TYPE_UNSPECIFIED\x10\x00\x12\n\n\x06NORMAL\x10\x01\x12\x08\n\x04\x46IFO\x10\x02\x12\t\n\x05\x44\x45LAY\x10\x03\x12\x0f\n\x0bTRANSACTION\x10\x04*G\n\nDigestType\x12\x1b\n\x17\x44IGEST_TYPE_UNSPECIFIED\x10\x00\x12\t\n\x05\x43RC32\x10\x01\x12\x07\n\x03MD5\x10\x02\x12\x08\n\x04SHA1\x10\x03*_\n\nClientType\x12\x1b\n\x17\x43LIENT_TYPE_UNSPECIFIED\x10\x00\x12\x0c\n\x08PRODUCER\x10\x01\x12\x11\n\rPUSH_CONSUMER\x10\x02\x12\x13\n\x0fSIMPLE_CONSUMER\x10\x03*<\n\x08\x45ncoding\x12\x18\n\x14\x45NCODING_UNSPECIFIED\x10\x00\x12\x0c\n\x08IDENTITY\x10\x01\x12\x08\n\x04GZIP\x10\x02*\xfe\t\n\x04\x43ode\x12\x14\n\x10\x43ODE_UNSPECIFIED\x10\x00\x12\x08\n\x02OK\x10\xa0\x9c\x01\x12\x16\n\x10MULTIPLE_RESULTS\x10\xb0\xea\x01\x12\x11\n\x0b\x42\x41\x44_REQUEST\x10\xc0\xb8\x02\x12\x1a\n\x14ILLEGAL_ACCESS_POINT\x10\xc1\xb8\x02\x12\x13\n\rILLEGAL_TOPIC\x10\xc2\xb8\x02\x12\x1c\n\x16ILLEGAL_CONSUMER_GROUP\x10\xc3\xb8\x02\x12\x19\n\x13ILLEGAL_MESSAGE_TAG\x10\xc4\xb8\x02\x12\x19\n\x13ILLEGAL_MESSAGE_KEY\x10\xc5\xb8\x02\x12\x1b\n\x15ILLEGAL_MESSAGE_GROUP\x10\xc6\xb8\x02\x12\"\n\x1cILLEGAL_MESSAGE_PROPERTY_KEY\x10\xc7\xb8\x02\x12\x1c\n\x16INVALID_TRANSACTION_ID\x10\xc8\xb8\x02\x12\x18\n\x12ILLEGAL_MESSAGE_ID\x10\xc9\xb8\x02\x12\x1f\n\x19ILLEGAL_FILTER_EXPRESSION\x10\xca\xb8\x02\x12\x1c\n\x16ILLEGAL_INVISIBLE_TIME\x10\xcb\xb8\x02\x12\x1b\n\x15ILLEGAL_DELIVERY_TIME\x10\xcc\xb8\x02\x12\x1c\n\x16INVALID_RECEIPT_HANDLE\x10\xcd\xb8\x02\x12)\n#MESSAGE_PROPERTY_CONFLICT_WITH_TYPE\x10\xce\xb8\x02\x12\x1e\n\x18UNRECOGNIZED_CLIENT_TYPE\x10\xcf\xb8\x02\x12\x17\n\x11MESSAGE_CORRUPTED\x10\xd0\xb8\x02\x12\x18\n\x12\x43LIENT_ID_REQUIRED\x10\xd1\xb8\x02\x12\x1a\n\x14ILLEGAL_POLLING_TIME\x10\xd2\xb8\x02\x12\x12\n\x0cUNAUTHORIZED\x10\xa4\xb9\x02\x12\x16\n\x10PAYMENT_REQUIRED\x10\x88\xba\x02\x12\x0f\n\tFORBIDDEN\x10\xec\xba\x02\x12\x0f\n\tNOT_FOUND\x10\xd0\xbb\x02\x12\x17\n\x11MESSAGE_NOT_FOUND\x10\xd1\xbb\x02\x12\x15\n\x0fTOPIC_NOT_FOUND\x10\xd2\xbb\x02\x12\x1e\n\x18\x43ONSUMER_GROUP_NOT_FOUND\x10\xd3\xbb\x02\x12\x15\n\x0fREQUEST_TIMEOUT\x10\xe0\xbe\x02\x12\x17\n\x11PAYLOAD_TOO_LARGE\x10\xd4\xc2\x02\x12\x1c\n\x16MESSAGE_BODY_TOO_LARGE\x10\xd5\xc2\x02\x12\x19\n\x13PRECONDITION_FAILED\x10\xb0\xce\x02\x12\x17\n\x11TOO_MANY_REQUESTS\x10\x94\xcf\x02\x12%\n\x1fREQUEST_HEADER_FIELDS_TOO_LARGE\x10\xdc\xd0\x02\x12\"\n\x1cMESSAGE_PROPERTIES_TOO_LARGE\x10\xdd\xd0\x02\x12\x14\n\x0eINTERNAL_ERROR\x10\xd0\x86\x03\x12\x1b\n\x15INTERNAL_SERVER_ERROR\x10\xd1\x86\x03\x12\x16\n\x10HA_NOT_AVAILABLE\x10\xd2\x86\x03\x12\x15\n\x0fNOT_IMPLEMENTED\x10\xb4\x87\x03\x12\x13\n\rPROXY_TIMEOUT\x10\xe0\x89\x03\x12 \n\x1aMASTER_PERSISTENCE_TIMEOUT\x10\xe1\x89\x03\x12\x1f\n\x19SLAVE_PERSISTENCE_TIMEOUT\x10\xe2\x89\x03\x12\x11\n\x0bUNSUPPORTED\x10\xc4\x8a\x03\x12\x19\n\x13VERSION_UNSUPPORTED\x10\xc5\x8a\x03\x12%\n\x1fVERIFY_FIFO_MESSAGE_UNSUPPORTED\x10\xc6\x8a\x03\x12\x1f\n\x19\x46\x41ILED_TO_CONSUME_MESSAGE\x10\xe0\xd4\x03*\xad\x01\n\x08Language\x12\x18\n\x14LANGUAGE_UNSPECIFIED\x10\x00\x12\x08\n\x04JAVA\x10\x01\x12\x07\n\x03\x43PP\x10\x02\x12\x0b\n\x07\x44OT_NET\x10\x03\x12\n\n\x06GOLANG\x10\x04\x12\x08\n\x04RUST\x10\x05\x12\n\n\x06PYTHON\x10\x06\x12\x07\n\x03PHP\x10\x07\x12\x0b\n\x07NODE_JS\x10\x08\x12\x08\n\x04RUBY\x10\t\x12\x0f\n\x0bOBJECTIVE_C\x10\n\x12\x08\n\x04\x44\x41RT\x10\x0b\x12\n\n\x06KOTLIN\x10\x0c\x42;\n\x12\x61pache.rocketmq.v2B\x08MQDomainP\x01\xa0\x01\x01\xd8\x01\x01\xaa\x02\x12\x41pache.Rocketmq.V2b\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'apache.rocketmq.v2.definition_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\022apache.rocketmq.v2B\010MQDomainP\001\240\001\001\330\001\001\252\002\022Apache.Rocketmq.V2'
  _MESSAGE_USERPROPERTIESENTRY._options = None
  _MESSAGE_USERPROPERTIESENTRY._serialized_options = b'8\001'
  _TRANSACTIONRESOLUTION._serialized_start=3962
  _TRANSACTIONRESOLUTION._serialized_end=4051
  _TRANSACTIONSOURCE._serialized_start=4053
  _TRANSACTIONSOURCE._serialized_end=4140
  _PERMISSION._serialized_start=4142
  _PERMISSION._serialized_end=4229
  _FILTERTYPE._serialized_start=4231
  _FILTERTYPE._serialized_end=4290
  _ADDRESSSCHEME._serialized_start=4292
  _ADDRESSSCHEME._serialized_end=4376
  _MESSAGETYPE._serialized_start=4378
  _MESSAGETYPE._serialized_end=4471
  _DIGESTTYPE._serialized_start=4473
  _DIGESTTYPE._serialized_end=4544
  _CLIENTTYPE._serialized_start=4546
  _CLIENTTYPE._serialized_end=4641
  _ENCODING._serialized_start=4643
  _ENCODING._serialized_end=4703
  _CODE._serialized_start=4706
  _CODE._serialized_end=5984
  _LANGUAGE._serialized_start=5987
  _LANGUAGE._serialized_end=6160
  _FILTEREXPRESSION._serialized_start=124
  _FILTEREXPRESSION._serialized_end=208
  _RETRYPOLICY._serialized_start=211
  _RETRYPOLICY._serialized_end=398
  _EXPONENTIALBACKOFF._serialized_start=400
  _EXPONENTIALBACKOFF._serialized_end=524
  _CUSTOMIZEDBACKOFF._serialized_start=526
  _CUSTOMIZEDBACKOFF._serialized_end=586
  _RESOURCE._serialized_start=588
  _RESOURCE._serialized_end=640
  _SUBSCRIPTIONENTRY._serialized_start=642
  _SUBSCRIPTIONENTRY._serialized_end=764
  _ADDRESS._serialized_start=766
  _ADDRESS._serialized_end=803
  _ENDPOINTS._serialized_start=805
  _ENDPOINTS._serialized_end=915
  _BROKER._serialized_start=917
  _BROKER._serialized_end=1001
  _MESSAGEQUEUE._serialized_start=1004
  _MESSAGEQUEUE._serialized_end=1234
  _DIGEST._serialized_start=1236
  _DIGEST._serialized_end=1308
  _SYSTEMPROPERTIES._serialized_start=1311
  _SYSTEMPROPERTIES._serialized_end=2350
  _DEADLETTERQUEUE._serialized_start=2352
  _DEADLETTERQUEUE._serialized_end=2404
  _MESSAGE._serialized_start=2407
  _MESSAGE._serialized_end=2669
  _MESSAGE_USERPROPERTIESENTRY._serialized_start=2616
  _MESSAGE_USERPROPERTIESENTRY._serialized_end=2669
  _ASSIGNMENT._serialized_start=2671
  _ASSIGNMENT._serialized_end=2740
  _STATUS._serialized_start=2742
  _STATUS._serialized_end=2807
  _UA._serialized_start=2809
  _UA._serialized_end=2914
  _SETTINGS._serialized_start=2917
  _SETTINGS._serialized_end=3445
  _PUBLISHING._serialized_start=3447
  _PUBLISHING._serialized_end=3559
  _SUBSCRIPTION._serialized_start=3562
  _SUBSCRIPTION._serialized_end=3869
  _METRIC._serialized_start=3871
  _METRIC._serialized_end=3960
# @@protoc_insertion_point(module_scope)



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import binascii
import gzip
import hashlib
from typing import Dict, List

from rocketmq.definition import MessageQueue
from rocketmq.protocol.definition_pb2 import DigestType as ProtoDigestType
from rocketmq.protocol.definition_pb2 import Encoding as ProtoEncoding


class Message:
    def __init__(
        self,
        topic: str,
        body: bytes,
        properties: map = None,
        tag: str = None,
        keys: str = None,
        message_group: str = None,
        delivery_timestamp: int = None,
    ):
        if properties is None:
            properties = {}
        self.__topic = topic
        self.__body = body
        self.__properties = properties
        self.__tag = tag
        self.__keys = keys
        self.__message_group = message_group
        self.__delivery_timestamp = delivery_timestamp

    @property
    def topic(self):
        return self.__topic

    @property
    def body(self):
        return self.__body

    @property
    def properties(self):
        return self.__properties

    @property
    def tag(self):
        return self.__tag

    @property
    def keys(self):
        return self.__keys

    @property
    def message_group(self):
        return self.__message_group

    @property
    def delivery_timestamp(self):
        return self.__delivery_timestamp


class MessageView:
    def __init__(
        self,
        message_id: str,
        topic: str,
        body: bytes,
        tag: str,
        message_group: str,
        delivery_timestamp: int,
        keys: List[str],
        properties: Dict[str, str],
        born_host: str,
        born_time: int,
        delivery_attempt: int,
        message_queue: MessageQueue,
        receipt_handle: str,
        offset: int,
        corrupted: bool
    ):
        self.__message_id = message_id
        self.__topic = topic
        self.__body = body
        self.__properties = properties
        self.__tag = tag
        self.__keys = keys
        self.__message_group = message_group
        self.__delivery_timestamp = delivery_timestamp
        self.__born_host = born_host
        self.__delivery_attempt = delivery_attempt
        self.__receipt_handle = receipt_handle
        self.__born_time = born_time
        self.__message_queue = message_queue
        self.__offset = offset
        self.__corrupted = corrupted

    @property
    def message_queue(self):
        return self.__message_queue

    @property
    def receipt_handle(self):
        return self.__receipt_handle

    @property
    def topic(self):
        return self.__topic

    @property
    def body(self):
        return self.__body

    @property
    def message_id(self):
        return self.__message_id

    @property
    def born_host(self):
        return self.__born_host

    @property
    def keys(self):
        return self.__keys

    @property
    def properties(self):
        return self.__properties

    @property
    def tag(self):
        return self.__tag

    @property
    def message_group(self):
        return self.__message_group

    @property
    def delivery_timestamp(self):
        return self.__delivery_timestamp

    @classmethod
    def from_protobuf(cls, message, message_queue=None):
        topic = message.topic.name
        system_properties = message.system_properties
        message_id = system_properties.message_id
        body_digest = system_properties.body_digest
        check_sum = body_digest.checksum
        raw = message.body
        corrupted = False
        digest_type = body_digest.type

        # Digest Type check
        if digest_type == ProtoDigestType.CRC32:
            expected_check_sum = format(binascii.crc32(raw) & 0xFFFFFFFF, '08X')
            if not expected_check_sum == check_sum:
                corrupted = True
        elif digest_type == ProtoDigestType.MD5:
            expected_check_sum = hashlib.md5(raw).hexdigest()
            if not expected_check_sum == check_sum:
                corrupted = True
        elif digest_type == ProtoDigestType.SHA1:
            expected_check_sum = hashlib.sha1(raw).hexdigest()
            if not expected_check_sum == check_sum:
                corrupted = True
        elif digest_type in [ProtoDigestType.unspecified, None]:
            print(f"Unsupported message body digest algorithm, digestType={digest_type}, topic={topic}, messageId={message_id}")

        # Body Encoding check
        body_encoding = system_properties.body_encoding
        body = raw
        if body_encoding == ProtoEncoding.GZIP:
            body = gzip.decompress(message.body)
        elif body_encoding in [ProtoEncoding.IDENTITY, None]:
            pass
        else:
            print(f"Unsupported message encoding algorithm, topic={topic}, messageId={message_id}, bodyEncoding={body_encoding}")

        tag = system_properties.tag
        message_group = system_properties.message_group
        delivery_time = system_properties.delivery_timestamp
        keys = list(system_properties.keys)

        born_host = system_properties.born_host
        born_time = system_properties.born_timestamp
        delivery_attempt = system_properties.delivery_attempt
        queue_offset = system_properties.queue_offset
        properties = {key: value for key, value in message.user_properties.items()}
        receipt_handle = system_properties.receipt_handle

        return cls(message_id, topic, body, tag, message_group, delivery_time, keys, properties, born_host,
                   born_time, delivery_attempt, message_queue, receipt_handle, queue_offset, corrupted)



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum
from typing import List

from protocol.definition_pb2 import Broker as ProtoBroker
from protocol.definition_pb2 import Encoding as ProtoEncoding
from protocol.definition_pb2 import MessageQueue as ProtoMessageQueue
from protocol.definition_pb2 import MessageType as ProtoMessageType
from protocol.definition_pb2 import Permission as ProtoPermission
from protocol.definition_pb2 import Resource as ProtoResource
from rocketmq.protocol import definition_pb2
from rocketmq.rpc_client import Endpoints


class Encoding(Enum):
    """Enumeration of supported encoding types."""
    IDENTITY = 0
    GZIP = 1


class EncodingHelper:
    """Helper class for converting encoding types to protobuf."""

    @staticmethod
    def to_protobuf(mq_encoding):
        """Convert encoding type to protobuf.

        :param mq_encoding: The encoding to be converted.
        :return: The corresponding protobuf encoding.
        """
        if mq_encoding == Encoding.IDENTITY:
            return ProtoEncoding.IDENTITY
        elif mq_encoding == Encoding.GZIP:
            return ProtoEncoding.GZIP


class Broker:
    """Represent a broker entity."""

    def __init__(self, broker):
        self.name = broker.name
        self.id = broker.id
        self.endpoints = Endpoints(broker.endpoints)

    def to_protobuf(self):
        """Convert the broker to its protobuf representation.

        :return: The protobuf representation of the broker.
        """
        return ProtoBroker(
            name=self.name, id=self.id, endpoints=self.endpoints.to_protobuf()
        )


class Resource:
    """Represent a resource entity."""

    def __init__(self, name=None, resource=None):
        """Initialize a resource.

        :param name: The name of the resource.
        :param resource: The resource object.
        """
        if resource is not None:
            self.namespace = resource.resource_namespace
            self.name = resource.name
        else:
            self.namespace = ""
            self.name = name

    def to_protobuf(self):
        """Convert the resource to its protobuf representation.

        :return: The protobuf representation of the resource.
        """
        resource = ProtoResource()
        resource.name = self.name
        resource.resource_namespace = self.namespace
        return resource

    def __str__(self):
        return f"{self.namespace}.{self.name}" if self.namespace else self.name


class Permission(Enum):
    """Enumeration of supported permission types."""
    NONE = 0
    READ = 1
    WRITE = 2
    READ_WRITE = 3


class PermissionHelper:
    """Helper class for converting permission types to protobuf and vice versa."""

    @staticmethod
    def from_protobuf(permission):
        """Convert protobuf permission to Permission enum.

        :param permission: The protobuf permission to be converted.
        :return: The corresponding Permission enum.
        """
        if permission == ProtoPermission.READ:
            return Permission.READ
        elif permission == ProtoPermission.WRITE:
            return Permission.WRITE
        elif permission == ProtoPermission.READ_WRITE:
            return Permission.READ_WRITE
        elif permission == ProtoPermission.NONE:
            return Permission.NONE
        else:
            pass

    @staticmethod
    def to_protobuf(permission):
        """Convert Permission enum to protobuf permission.

        :param permission: The Permission enum to be converted.
        :return: The corresponding protobuf permission.
        """
        if permission == Permission.READ:
            return ProtoPermission.READ
        elif permission == Permission.WRITE:
            return ProtoPermission.WRITE
        elif permission == Permission.READ_WRITE:
            return ProtoPermission.READ_WRITE
        else:
            pass

    @staticmethod
    def is_writable(permission):
        """Check if the permission is writable.

        :param permission: The Permission enum to be checked.
        :return: True if the permission is writable, False otherwise.
        """
        if permission in [Permission.WRITE, Permission.READ_WRITE]:
            return True
        else:
            return False

    @staticmethod
    def is_readable(permission):
        """Check if the permission is readable.

        :param permission: The Permission enum to be checked.
        :return: True if the permission is readable, False otherwise.
        """
        if permission in [Permission.READ, Permission.READ_WRITE]:
            return True
        else:
            return False


class MessageType(Enum):
    """Enumeration of supported message types."""
    NORMAL = 0
    FIFO = 1
    DELAY = 2
    TRANSACTION = 3


class MessageTypeHelper:
    """Helper class for converting message types to protobuf and vice versa."""

    @staticmethod
    def from_protobuf(message_type):
        """Convert protobuf message type to MessageType enum.

        :param message_type: The protobuf message type to be converted.
        :return: The corresponding MessageType enum.
        """
        if message_type == ProtoMessageType.NORMAL:
            return MessageType.NORMAL
        elif message_type == ProtoMessageType.FIFO:
            return MessageType.FIFO
        elif message_type == ProtoMessageType.DELAY:
            return MessageType.DELAY
        elif message_type == ProtoMessageType.TRANSACTION:
            return MessageType.TRANSACTION
        else:
            pass

    @staticmethod
    def to_protobuf(message_type):
        """Convert MessageType enum to protobuf message type.

        :param message_type: The MessageType enum to be converted.
        :return: The corresponding protobuf message type.
        """
        if message_type == MessageType.NORMAL:
            return ProtoMessageType.NORMAL
        elif message_type == MessageType.FIFO:
            return ProtoMessageType.FIFO
        elif message_type == MessageType.DELAY:
            return ProtoMessageType.DELAY
        elif message_type == MessageType.TRANSACTION:
            return ProtoMessageType.TRANSACTION
        else:
            return ProtoMessageType.UNSPECIFIED


class MessageQueue:
    """A class that encapsulates a message queue entity."""

    def __init__(self, message_queue):
        """Initialize a MessageQueue instance.

        :param message_queue: The initial message queue to be encapsulated.
        """
        self._topic_resource = Resource(message_queue.topic.name, message_queue.topic)
        self.queue_id = message_queue.id
        self.permission = PermissionHelper.from_protobuf(message_queue.permission)
        self.accept_message_types = [
            MessageTypeHelper.from_protobuf(mt)
            for mt in message_queue.accept_message_types
        ]
        self.broker = Broker(message_queue.broker)

    @property
    def topic(self):
        """The topic resource name.

        :return: The name of the topic resource.
        """
        return self._topic_resource.name

    def __str__(self):
        """Get a string representation of the MessageQueue instance.

        :return: A string that represents the MessageQueue instance.
        """
        return f"{self.broker.name}.{self._topic_resource}.{self.queue_id}"

    def to_protobuf(self):
        """Convert the MessageQueue instance to protobuf message queue.

        :return: A protobuf message queue that represents the MessageQueue instance.
        """
        message_types = [
            MessageTypeHelper.to_protobuf(mt) for mt in self.accept_message_types
        ]
        return ProtoMessageQueue(
            topic=self._topic_resource.to_protobuf(),
            id=self.queue_id,
            permission=PermissionHelper.to_protobuf(self.permission),
            broker=self.broker.to_protobuf(),
            accept_message_types=message_types,
        )


class TopicRouteData:
    """A class that encapsulates a list of message queues."""

    def __init__(self, message_queues: List[definition_pb2.MessageQueue]):
        """Initialize a TopicRouteData instance.

        :param message_queues: The initial list of message queues to be encapsulated.
        """
        message_queue_list = []
        for mq in message_queues:
            message_queue_list.append(MessageQueue(mq))
        self.__message_queue_list = message_queue_list

    @property
    def message_queues(self) -> List[MessageQueue]:
        """The list of MessageQueue instances.

        :return: The list of MessageQueue instances that the TopicRouteData instance encapsulates.
        """
        return self.__message_queue_list



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: apache/rocketmq/v2/service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from protocol import definition_pb2 as apache_dot_rocketmq_dot_v2_dot_definition__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n apache/rocketmq/v2/service.proto\x12\x12\x61pache.rocketmq.v2\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a#apache/rocketmq/v2/definition.proto\"r\n\x11QueryRouteRequest\x12+\n\x05topic\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12\x30\n\tendpoints\x18\x02 \x01(\x0b\x32\x1d.apache.rocketmq.v2.Endpoints\"z\n\x12QueryRouteResponse\x12*\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\x12\x38\n\x0emessage_queues\x18\x02 \x03(\x0b\x32 .apache.rocketmq.v2.MessageQueue\"C\n\x12SendMessageRequest\x12-\n\x08messages\x18\x01 \x03(\x0b\x32\x1b.apache.rocketmq.v2.Message\"y\n\x0fSendResultEntry\x12*\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\x12\x12\n\nmessage_id\x18\x02 \x01(\t\x12\x16\n\x0etransaction_id\x18\x03 \x01(\t\x12\x0e\n\x06offset\x18\x04 \x01(\x03\"w\n\x13SendMessageResponse\x12*\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\x12\x34\n\x07\x65ntries\x18\x02 \x03(\x0b\x32#.apache.rocketmq.v2.SendResultEntry\"\xa4\x01\n\x16QueryAssignmentRequest\x12+\n\x05topic\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12+\n\x05group\x18\x02 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12\x30\n\tendpoints\x18\x03 \x01(\x0b\x32\x1d.apache.rocketmq.v2.Endpoints\"z\n\x17QueryAssignmentResponse\x12*\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\x12\x33\n\x0b\x61ssignments\x18\x02 \x03(\x0b\x32\x1e.apache.rocketmq.v2.Assignment\"\x90\x03\n\x15ReceiveMessageRequest\x12+\n\x05group\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12\x37\n\rmessage_queue\x18\x02 \x01(\x0b\x32 .apache.rocketmq.v2.MessageQueue\x12?\n\x11\x66ilter_expression\x18\x03 \x01(\x0b\x32$.apache.rocketmq.v2.FilterExpression\x12\x12\n\nbatch_size\x18\x04 \x01(\x05\x12:\n\x12invisible_duration\x18\x05 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00\x88\x01\x01\x12\x12\n\nauto_renew\x18\x06 \x01(\x08\x12<\n\x14long_polling_timeout\x18\x07 \x01(\x0b\x32\x19.google.protobuf.DurationH\x01\x88\x01\x01\x42\x15\n\x13_invisible_durationB\x17\n\x15_long_polling_timeout\"\xbb\x01\n\x16ReceiveMessageResponse\x12,\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.StatusH\x00\x12.\n\x07message\x18\x02 \x01(\x0b\x32\x1b.apache.rocketmq.v2.MessageH\x00\x12\x38\n\x12\x64\x65livery_timestamp\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x42\t\n\x07\x63ontent\"=\n\x0f\x41\x63kMessageEntry\x12\x12\n\nmessage_id\x18\x01 \x01(\t\x12\x16\n\x0ereceipt_handle\x18\x02 \x01(\t\"\xa3\x01\n\x11\x41\x63kMessageRequest\x12+\n\x05group\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12+\n\x05topic\x18\x02 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12\x34\n\x07\x65ntries\x18\x03 \x03(\x0b\x32#.apache.rocketmq.v2.AckMessageEntry\"o\n\x15\x41\x63kMessageResultEntry\x12\x12\n\nmessage_id\x18\x01 \x01(\t\x12\x16\n\x0ereceipt_handle\x18\x02 \x01(\t\x12*\n\x06status\x18\x03 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\"|\n\x12\x41\x63kMessageResponse\x12*\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\x12:\n\x07\x65ntries\x18\x02 \x03(\x0b\x32).apache.rocketmq.v2.AckMessageResultEntry\"\xe7\x01\n&ForwardMessageToDeadLetterQueueRequest\x12+\n\x05group\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12+\n\x05topic\x18\x02 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12\x16\n\x0ereceipt_handle\x18\x03 \x01(\t\x12\x12\n\nmessage_id\x18\x04 \x01(\t\x12\x18\n\x10\x64\x65livery_attempt\x18\x05 \x01(\x05\x12\x1d\n\x15max_delivery_attempts\x18\x06 \x01(\x05\"U\n\'ForwardMessageToDeadLetterQueueResponse\x12*\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\"\x83\x01\n\x10HeartbeatRequest\x12\x30\n\x05group\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.ResourceH\x00\x88\x01\x01\x12\x33\n\x0b\x63lient_type\x18\x02 \x01(\x0e\x32\x1e.apache.rocketmq.v2.ClientTypeB\x08\n\x06_group\"?\n\x11HeartbeatResponse\x12*\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\"\xfd\x01\n\x15\x45ndTransactionRequest\x12+\n\x05topic\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12\x12\n\nmessage_id\x18\x02 \x01(\t\x12\x16\n\x0etransaction_id\x18\x03 \x01(\t\x12=\n\nresolution\x18\x04 \x01(\x0e\x32).apache.rocketmq.v2.TransactionResolution\x12\x35\n\x06source\x18\x05 \x01(\x0e\x32%.apache.rocketmq.v2.TransactionSource\x12\x15\n\rtrace_context\x18\x06 \x01(\t\"D\n\x16\x45ndTransactionResponse\x12*\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\"-\n\x1cPrintThreadStackTraceCommand\x12\r\n\x05nonce\x18\x01 \x01(\t\"Y\n\x10ThreadStackTrace\x12\r\n\x05nonce\x18\x01 \x01(\t\x12\x1f\n\x12thread_stack_trace\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x15\n\x13_thread_stack_trace\"S\n\x14VerifyMessageCommand\x12\r\n\x05nonce\x18\x01 \x01(\t\x12,\n\x07message\x18\x02 \x01(\x0b\x32\x1b.apache.rocketmq.v2.Message\"$\n\x13VerifyMessageResult\x12\r\n\x05nonce\x18\x01 \x01(\t\"i\n!RecoverOrphanedTransactionCommand\x12,\n\x07message\x18\x01 \x01(\x0b\x32\x1b.apache.rocketmq.v2.Message\x12\x16\n\x0etransaction_id\x18\x02 \x01(\t\"\xaa\x04\n\x10TelemetryCommand\x12/\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.StatusH\x01\x88\x01\x01\x12\x30\n\x08settings\x18\x02 \x01(\x0b\x32\x1c.apache.rocketmq.v2.SettingsH\x00\x12\x42\n\x12thread_stack_trace\x18\x03 \x01(\x0b\x32$.apache.rocketmq.v2.ThreadStackTraceH\x00\x12H\n\x15verify_message_result\x18\x04 \x01(\x0b\x32\'.apache.rocketmq.v2.VerifyMessageResultH\x00\x12\x65\n$recover_orphaned_transaction_command\x18\x05 \x01(\x0b\x32\x35.apache.rocketmq.v2.RecoverOrphanedTransactionCommandH\x00\x12\\\n print_thread_stack_trace_command\x18\x06 \x01(\x0b\x32\x30.apache.rocketmq.v2.PrintThreadStackTraceCommandH\x00\x12J\n\x16verify_message_command\x18\x07 \x01(\x0b\x32(.apache.rocketmq.v2.VerifyMessageCommandH\x00\x42\t\n\x07\x63ommandB\t\n\x07_status\"\\\n\x1eNotifyClientTerminationRequest\x12\x30\n\x05group\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.ResourceH\x00\x88\x01\x01\x42\x08\n\x06_group\"M\n\x1fNotifyClientTerminationResponse\x12*\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\"\xdd\x01\n\x1e\x43hangeInvisibleDurationRequest\x12+\n\x05group\x18\x01 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12+\n\x05topic\x18\x02 \x01(\x0b\x32\x1c.apache.rocketmq.v2.Resource\x12\x16\n\x0ereceipt_handle\x18\x03 \x01(\t\x12\x35\n\x12invisible_duration\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x12\n\nmessage_id\x18\x05 \x01(\t\"e\n\x1f\x43hangeInvisibleDurationResponse\x12*\n\x06status\x18\x01 \x01(\x0b\x32\x1a.apache.rocketmq.v2.Status\x12\x16\n\x0ereceipt_handle\x18\x02 \x01(\t2\xe0\t\n\x10MessagingService\x12]\n\nQueryRoute\x12%.apache.rocketmq.v2.QueryRouteRequest\x1a&.apache.rocketmq.v2.QueryRouteResponse\"\x00\x12Z\n\tHeartbeat\x12$.apache.rocketmq.v2.HeartbeatRequest\x1a%.apache.rocketmq.v2.HeartbeatResponse\"\x00\x12`\n\x0bSendMessage\x12&.apache.rocketmq.v2.SendMessageRequest\x1a\'.apache.rocketmq.v2.SendMessageResponse\"\x00\x12l\n\x0fQueryAssignment\x12*.apache.rocketmq.v2.QueryAssignmentRequest\x1a+.apache.rocketmq.v2.QueryAssignmentResponse\"\x00\x12k\n\x0eReceiveMessage\x12).apache.rocketmq.v2.ReceiveMessageRequest\x1a*.apache.rocketmq.v2.ReceiveMessageResponse\"\x00\x30\x01\x12]\n\nAckMessage\x12%.apache.rocketmq.v2.AckMessageRequest\x1a&.apache.rocketmq.v2.AckMessageResponse\"\x00\x12\x9c\x01\n\x1f\x46orwardMessageToDeadLetterQueue\x12:.apache.rocketmq.v2.ForwardMessageToDeadLetterQueueRequest\x1a;.apache.rocketmq.v2.ForwardMessageToDeadLetterQueueResponse\"\x00\x12i\n\x0e\x45ndTransaction\x12).apache.rocketmq.v2.EndTransactionRequest\x1a*.apache.rocketmq.v2.EndTransactionResponse\"\x00\x12]\n\tTelemetry\x12$.apache.rocketmq.v2.TelemetryCommand\x1a$.apache.rocketmq.v2.TelemetryCommand\"\x00(\x01\x30\x01\x12\x84\x01\n\x17NotifyClientTermination\x12\x32.apache.rocketmq.v2.NotifyClientTerminationRequest\x1a\x33.apache.rocketmq.v2.NotifyClientTerminationResponse\"\x00\x12\x84\x01\n\x17\x43hangeInvisibleDuration\x12\x32.apache.rocketmq.v2.ChangeInvisibleDurationRequest\x1a\x33.apache.rocketmq.v2.ChangeInvisibleDurationResponse\"\x00\x42<\n\x12\x61pache.rocketmq.v2B\tMQServiceP\x01\xa0\x01\x01\xd8\x01\x01\xaa\x02\x12\x41pache.Rocketmq.V2b\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'apache.rocketmq.v2.service_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\022apache.rocketmq.v2B\tMQServiceP\001\240\001\001\330\001\001\252\002\022Apache.Rocketmq.V2'
  _QUERYROUTEREQUEST._serialized_start=158
  _QUERYROUTEREQUEST._serialized_end=272
  _QUERYROUTERESPONSE._serialized_start=274
  _QUERYROUTERESPONSE._serialized_end=396
  _SENDMESSAGEREQUEST._serialized_start=398
  _SENDMESSAGEREQUEST._serialized_end=465
  _SENDRESULTENTRY._serialized_start=467
  _SENDRESULTENTRY._serialized_end=588
  _SENDMESSAGERESPONSE._serialized_start=590
  _SENDMESSAGERESPONSE._serialized_end=709
  _QUERYASSIGNMENTREQUEST._serialized_start=712
  _QUERYASSIGNMENTREQUEST._serialized_end=876
  _QUERYASSIGNMENTRESPONSE._serialized_start=878
  _QUERYASSIGNMENTRESPONSE._serialized_end=1000
  _RECEIVEMESSAGEREQUEST._serialized_start=1003
  _RECEIVEMESSAGEREQUEST._serialized_end=1403
  _RECEIVEMESSAGERESPONSE._serialized_start=1406
  _RECEIVEMESSAGERESPONSE._serialized_end=1593
  _ACKMESSAGEENTRY._serialized_start=1595
  _ACKMESSAGEENTRY._serialized_end=1656
  _ACKMESSAGEREQUEST._serialized_start=1659
  _ACKMESSAGEREQUEST._serialized_end=1822
  _ACKMESSAGERESULTENTRY._serialized_start=1824
  _ACKMESSAGERESULTENTRY._serialized_end=1935
  _ACKMESSAGERESPONSE._serialized_start=1937
  _ACKMESSAGERESPONSE._serialized_end=2061
  _FORWARDMESSAGETODEADLETTERQUEUEREQUEST._serialized_start=2064
  _FORWARDMESSAGETODEADLETTERQUEUEREQUEST._serialized_end=2295
  _FORWARDMESSAGETODEADLETTERQUEUERESPONSE._serialized_start=2297
  _FORWARDMESSAGETODEADLETTERQUEUERESPONSE._serialized_end=2382
  _HEARTBEATREQUEST._serialized_start=2385
  _HEARTBEATREQUEST._serialized_end=2516
  _HEARTBEATRESPONSE._serialized_start=2518
  _HEARTBEATRESPONSE._serialized_end=2581
  _ENDTRANSACTIONREQUEST._serialized_start=2584
  _ENDTRANSACTIONREQUEST._serialized_end=2837
  _ENDTRANSACTIONRESPONSE._serialized_start=2839
  _ENDTRANSACTIONRESPONSE._serialized_end=2907
  _PRINTTHREADSTACKTRACECOMMAND._serialized_start=2909
  _PRINTTHREADSTACKTRACECOMMAND._serialized_end=2954
  _THREADSTACKTRACE._serialized_start=2956
  _THREADSTACKTRACE._serialized_end=3045
  _VERIFYMESSAGECOMMAND._serialized_start=3047
  _VERIFYMESSAGECOMMAND._serialized_end=3130
  _VERIFYMESSAGERESULT._serialized_start=3132
  _VERIFYMESSAGERESULT._serialized_end=3168
  _RECOVERORPHANEDTRANSACTIONCOMMAND._serialized_start=3170
  _RECOVERORPHANEDTRANSACTIONCOMMAND._serialized_end=3275
  _TELEMETRYCOMMAND._serialized_start=3278
  _TELEMETRYCOMMAND._serialized_end=3832
  _NOTIFYCLIENTTERMINATIONREQUEST._serialized_start=3834
  _NOTIFYCLIENTTERMINATIONREQUEST._serialized_end=3926
  _NOTIFYCLIENTTERMINATIONRESPONSE._serialized_start=3928
  _NOTIFYCLIENTTERMINATIONRESPONSE._serialized_end=4005
  _CHANGEINVISIBLEDURATIONREQUEST._serialized_start=4008
  _CHANGEINVISIBLEDURATIONREQUEST._serialized_end=4229
  _CHANGEINVISIBLEDURATIONRESPONSE._serialized_start=4231
  _CHANGEINVISIBLEDURATIONRESPONSE._serialized_end=4332
  _MESSAGINGSERVICE._serialized_start=4335
  _MESSAGINGSERVICE._serialized_end=5583
# @@protoc_insertion_point(module_scope)



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import threading
import time
# from status_checker import StatusChecker
from datetime import datetime, timedelta
from threading import RLock
from typing import Set
from unittest.mock import MagicMock, patch

import rocketmq
from publishing_message import MessageType
from rocketmq.client import Client
from rocketmq.client_config import ClientConfig
from rocketmq.definition import PermissionHelper, TopicRouteData
from rocketmq.exponential_backoff_retry_policy import \
    ExponentialBackoffRetryPolicy
from rocketmq.log import logger
from rocketmq.message import Message
from rocketmq.message_id_codec import MessageIdCodec
from rocketmq.protocol.definition_pb2 import Message as ProtoMessage
from rocketmq.protocol.definition_pb2 import Resource
from rocketmq.protocol.definition_pb2 import Resource as ProtoResource
from rocketmq.protocol.definition_pb2 import SystemProperties
from rocketmq.protocol.definition_pb2 import \
    TransactionResolution as ProtoTransactionResolution
from rocketmq.protocol.service_pb2 import (EndTransactionRequest,
                                           SendMessageRequest)
from rocketmq.publish_settings import PublishingSettings
from rocketmq.publishing_message import PublishingMessage
from rocketmq.rpc_client import Endpoints
from rocketmq.send_receipt import SendReceipt
from rocketmq.session_credentials import (SessionCredentials,
                                          SessionCredentialsProvider)
from status_checker import TooManyRequestsException
from utils import get_positive_mod


class Transaction:
    MAX_MESSAGE_NUM = 1

    def __init__(self, producer):
        self.producer = producer
        self.messages = set()
        self.messages_lock = RLock()
        self.message_send_receipt_dict = {}

    def try_add_message(self, message):
        with self.messages_lock:
            if len(self.messages) > self.MAX_MESSAGE_NUM:
                raise ValueError(f"Message in transaction has exceed the threshold: {self.MAX_MESSAGE_NUM}")

            publishing_message = PublishingMessage(message, self.producer.publish_settings, True)
            self.messages.add(publishing_message)
            return publishing_message

    def try_add_receipt(self, publishing_message, send_receipt):
        with self.messages_lock:
            if publishing_message not in self.messages:
                raise ValueError("Message is not in the transaction")

            self.message_send_receipt_dict[publishing_message] = send_receipt

    async def commit(self):
        # if self.producer.state != "Running":
        #     raise Exception("Producer is not running")

        if not self.message_send_receipt_dict:
            raise ValueError("Transactional message has not been sent yet")

        for publishing_message, send_receipt in self.message_send_receipt_dict.items():
            await self.producer.end_transaction(send_receipt.endpoints, publishing_message.message.topic, send_receipt.message_id, send_receipt.transaction_id, "Commit")

    async def rollback(self):
        # if self.producer.state != "Running":
        #     raise Exception("Producer is not running")

        if not self.message_send_receipt_dict:
            raise ValueError("Transactional message has not been sent yet")

        for publishing_message, send_receipt in self.message_send_receipt_dict.items():
            await self.producer.end_transaction(send_receipt.endpoints, publishing_message.message.topic, send_receipt.message_id, send_receipt.transaction_id, "Rollback")


class PublishingLoadBalancer:
    """This class serves as a load balancer for message publishing.
    It keeps track of a rotating index to help distribute the load evenly.
    """

    def __init__(self, topic_route_data: TopicRouteData, index: int = 0):
        #: current index for message queue selection
        self.__index = index
        #: thread lock to ensure atomic update to the index
        self.__index_lock = threading.Lock()

        #: filter the message queues which are writable and from the master broker
        message_queues = []
        for mq in topic_route_data.message_queues:
            if (
                not PermissionHelper().is_writable(mq.permission)
                or mq.broker.id is not rocketmq.utils.master_broker_id
            ):
                continue
            message_queues.append(mq)
        self.__message_queues = message_queues

    @property
    def index(self):
        """Property to fetch the current index"""
        return self.__index

    def get_and_increment_index(self):
        """Thread safe method to get the current index and increment it by one"""
        with self.__index_lock:
            temp = self.__index
            self.__index += 1
            return temp

    def take_message_queues(self, excluded: Set[Endpoints], count: int):
        """Fetch a specified number of message queues, excluding the ones provided.
        It will first try to fetch from non-excluded brokers and if insufficient,
        it will select from the excluded ones.
        """
        next_index = self.get_and_increment_index()
        candidates = []
        candidate_broker_name = set()

        queue_num = len(self.__message_queues)
        for i in range(queue_num):
            mq = self.__message_queues[next_index % queue_num]
            next_index = next_index + 1
            if (
                mq.broker.endpoints not in excluded
                and mq.broker.name not in candidate_broker_name
            ):
                candidate_broker_name.add(mq.broker.name)
                candidates.append(mq)
            if len(candidates) >= count:
                return candidates
        # if all endpoints are isolated
        if candidates:
            return candidates
        for i in range(queue_num):
            mq = self.__message_queues[next_index % queue_num]
            if mq.broker.name not in candidate_broker_name:
                candidate_broker_name.add(mq.broker.name)
                candidates.append(mq)
            if len(candidates) >= count:
                return candidates
        return candidates

    def take_message_queue_by_message_group(self, message_group):
        index = get_positive_mod(hash(message_group), len(self.__message_queues))
        return self.__message_queues[index]


class Producer(Client):
    """The Producer class extends the Client class and is used to publish
    messages to specific topics in RocketMQ.
    """

    def __init__(self, client_config: ClientConfig, topics: Set[str]):
        """Create a new Producer.

        :param client_config: The configuration for the client.
        :param topics: The set of topics to which the producer can send messages.
        """
        super().__init__(client_config)
        self.publish_topics = topics
        retry_policy = ExponentialBackoffRetryPolicy.immediately_retry_policy(10)
        #: Set up the publishing settings with the given parameters.
        self.publish_settings = PublishingSettings(
            self.client_id, self.endpoints, retry_policy, 10, topics
        )
        #: Initialize the routedata cache.
        self.publish_routedata_cache = {}

    async def __aenter__(self):
        """Provide an asynchronous context manager for the producer."""
        await self.start()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Provide an asynchronous context manager for the producer."""
        await self.shutdown()

    def get_topics(self):
        return self.publish_topics

    async def start(self):
        """Start the RocketMQ producer and log the operation."""
        logger.info(f"Begin to start the rocketmq producer, client_id={self.client_id}")
        await super().start()
        logger.info(f"The rocketmq producer starts successfully, client_id={self.client_id}")

    async def shutdown(self):
        """Shutdown the RocketMQ producer and log the operation."""
        logger.info(f"Begin to shutdown the rocketmq producer, client_id={self.client_id}")
        await super().shutdown()
        logger.info(f"Shutdown the rocketmq producer successfully, client_id={self.client_id}")

    @staticmethod
    def wrap_send_message_request(message, message_queue):
        """Wrap the send message request for the RocketMQ producer.

        :param message: The message to be sent.
        :param message_queue: The queue to which the message will be sent.
        :return: The SendMessageRequest with the message and queue details.
        """
        req = SendMessageRequest()
        req.messages.extend([message.to_protobuf(message_queue.queue_id)])
        return req

    async def send(self, message, transaction: Transaction = None):
        tx_enabled = True
        if transaction is None:
            tx_enabled = False
        if tx_enabled:
            logger.debug("Transaction send")
            publishing_message = transaction.try_add_message(message)
            send_receipt = await self.send_message(message, tx_enabled)
            transaction.try_add_receipt(publishing_message, send_receipt)
            return send_receipt
        else:
            return await self.send_message(message)

    async def send_message(self, message, tx_enabled=False):
        """Send a message using a load balancer, retrying as needed according to the retry policy.

        :param message: The message to be sent.
        """
        publish_load_balancer = await self.get_publish_load_balancer(message.topic)
        publishing_message = PublishingMessage(message, self.publish_settings, tx_enabled)
        retry_policy = self.get_retry_policy()
        max_attempts = retry_policy.get_max_attempts()

        exception = None
        logger.debug(publishing_message.message.message_group)
        candidates = (
            publish_load_balancer.take_message_queues(set(self.isolated.keys()), max_attempts)
            if publishing_message.message.message_group is None else
            [publish_load_balancer.take_message_queue_by_message_group(publishing_message.message.message_group)])
        for attempt in range(1, max_attempts + 1):
            start_time = time.time()
            candidate_index = (attempt - 1) % len(candidates)
            mq = candidates[candidate_index]
            logger.debug(mq.accept_message_types)
            if self.publish_settings.is_validate_message_type() and publishing_message.message_type.value != mq.accept_message_types[0].value:
                raise ValueError(
                    "Current message type does not match with the accept message types,"
                    + f" topic={message.topic}, actualMessageType={publishing_message.message_type}"
                    + f" acceptMessageType={','}")

            send_message_request = self.wrap_send_message_request(publishing_message, mq)
            # topic_data = self.topic_route_cache["normal_topic"]
            endpoints = mq.broker.endpoints

            try:
                invocation = await self.client_manager.send_message(endpoints, send_message_request, self.client_config.request_timeout)
                logger.debug(invocation)
                send_recepits = SendReceipt.process_send_message_response(mq, invocation)
                send_recepit = send_recepits[0]
                if attempt > 1:
                    logger.info(
                        f"Re-send message successfully, topic={message.topic},"
                        + f" max_attempts={max_attempts}, endpoints={str(endpoints)}, clientId={self.client_id}")
                return send_recepit
            except Exception as e:
                exception = e
                self.isolated[endpoints] = True
                if attempt >= max_attempts:
                    logger.error("Failed to send message finally, run out of attempt times, "
                                 + f"topic={message.topic}, maxAttempt={max_attempts}, attempt={attempt}, "
                                 + f"endpoints={endpoints}, messageId={publishing_message.message_id}, clientId={self.client_id}")
                    raise
                if publishing_message.message_type == MessageType.TRANSACTION:
                    logger.error("Failed to send transaction message, run out of attempt times, "
                                 + f"topic={message.topic}, maxAttempt=1, attempt={attempt}, "
                                 + f"endpoints={endpoints}, messageId={publishing_message.message_id}, clientId={self.client_id}")
                    raise
                if not isinstance(exception, TooManyRequestsException):
                    logger.error(f"Failed to send message, topic={message.topic}, max_attempts={max_attempts}, "
                                 + f"attempt={attempt}, endpoints={endpoints}, messageId={publishing_message.message_id},"
                                 + f" clientId={self.client_id}")
                    continue

                nextAttempt = 1 + attempt
                delay = retry_policy.get_next_attempt_delay(nextAttempt)
                await asyncio.sleep(delay.total_seconds())
                logger.warning(f"Failed to send message due to too many requests, would attempt to resend after {delay},\
                                 topic={message.topic}, max_attempts={max_attempts}, attempt={attempt}, endpoints={endpoints},\
                                 message_id={publishing_message.message_id}, client_id={self.client_id}")
            finally:
                elapsed_time = time.time() - start_time
                logger.info(f"send time: {elapsed_time}")

    def update_publish_load_balancer(self, topic, topic_route_data):
        """Update the load balancer used for publishing messages to a topic.

        :param topic: The topic for which to update the load balancer.
        :param topic_route_data: The new route data for the topic.
        :return: The updated load balancer.
        """
        publishing_load_balancer = None
        if topic in self.publish_routedata_cache:
            publishing_load_balancer = self.publish_routedata_cache[topic]
        else:
            publishing_load_balancer = PublishingLoadBalancer(topic_route_data)
        self.publish_routedata_cache[topic] = publishing_load_balancer
        return publishing_load_balancer

    async def get_publish_load_balancer(self, topic):
        """Get the load balancer used for publishing messages to a topic.

        :param topic: The topic for which to get the load balancer.
        :return: The load balancer for the topic.
        """
        if topic in self.publish_routedata_cache:
            return self.publish_routedata_cache[topic]
        topic_route_data = await self.get_route_data(topic)
        return self.update_publish_load_balancer(topic, topic_route_data)

    def get_settings(self):
        """Get the publishing settings for this producer.

        :return: The publishing settings for this producer.
        """
        return self.publish_settings

    def get_retry_policy(self):
        """Get the retry policy for this producer.

        :return: The retry policy for this producer.
        """
        return self.publish_settings.GetRetryPolicy()

    def begin_transaction(self):
        """Start a new transaction."""
        return Transaction(self)

    async def end_transaction(self, endpoints, topic, message_id, transaction_id, resolution):
        """End a transaction based on its resolution (commit or rollback)."""
        topic_resource = ProtoResource(name=topic)
        request = EndTransactionRequest(
            transaction_id=transaction_id,
            message_id=message_id,
            topic=topic_resource,
            resolution=ProtoTransactionResolution.COMMIT if resolution == "Commit" else ProtoTransactionResolution.ROLLBACK
        )
        await self.client_manager.end_transaction(endpoints, request, self.client_config.request_timeout)
        # StatusChecker.check(invocation.response.status, request, invocation.request_id)


async def test():
    credentials = SessionCredentials("username", "password")
    credentials_provider = SessionCredentialsProvider(credentials)
    client_config = ClientConfig(
        endpoints=Endpoints("endpoint"),
        session_credentials_provider=credentials_provider,
        ssl_enabled=True,
    )
    topic = Resource()
    topic.name = "normal_topic"
    msg = ProtoMessage()
    msg.topic.CopyFrom(topic)
    msg.body = b"My Normal Message Body"
    sysperf = SystemProperties()
    sysperf.message_id = MessageIdCodec.next_message_id()
    sysperf.message_group = "yourConsumerGroup"
    msg.system_properties.CopyFrom(sysperf)
    producer = Producer(client_config, topics={"normal_topic"})
    message = Message(topic.name, msg.body)
    await producer.start()
    await asyncio.sleep(10)
    send_receipt = await producer.send(message)
    logger.info(f"Send message successfully, {send_receipt}")


async def test_delay_message():
    credentials = SessionCredentials("username", "password")
    credentials_provider = SessionCredentialsProvider(credentials)
    client_config = ClientConfig(
        endpoints=Endpoints("endpoint"),
        session_credentials_provider=credentials_provider,
        ssl_enabled=True,
    )
    topic = Resource()
    topic.name = "delay_topic"
    msg = ProtoMessage()
    msg.topic.CopyFrom(topic)
    msg.body = b"My Delay Message Body"
    sysperf = SystemProperties()
    sysperf.message_id = MessageIdCodec.next_message_id()
    msg.system_properties.CopyFrom(sysperf)
    logger.debug(f"{msg}")
    producer = Producer(client_config, topics={"delay_topic"})
    current_time_millis = int(round(time.time() * 1000))
    message_delay_time = timedelta(seconds=10)
    result_time_millis = current_time_millis + int(message_delay_time.total_seconds() * 1000)
    result_time_datetime = datetime.fromtimestamp(result_time_millis / 1000.0)
    message = Message(topic.name, msg.body, delivery_timestamp=result_time_datetime)
    await producer.start()
    await asyncio.sleep(10)
    send_receipt = await producer.send(message)
    logger.info(f"Send message successfully, {send_receipt}")


async def test_fifo_message():
    credentials = SessionCredentials("username", "password")
    credentials_provider = SessionCredentialsProvider(credentials)
    client_config = ClientConfig(
        endpoints=Endpoints("endpoint"),
        session_credentials_provider=credentials_provider,
        ssl_enabled=True,
    )
    topic = Resource()
    topic.name = "fifo_topic"
    msg = ProtoMessage()
    msg.topic.CopyFrom(topic)
    msg.body = b"My FIFO Message Body"
    sysperf = SystemProperties()
    sysperf.message_id = MessageIdCodec.next_message_id()
    msg.system_properties.CopyFrom(sysperf)
    logger.debug(f"{msg}")
    producer = Producer(client_config, topics={"fifo_topic"})
    message = Message(topic.name, msg.body, message_group="yourConsumerGroup")
    await producer.start()
    await asyncio.sleep(10)
    send_receipt = await producer.send(message)
    logger.info(f"Send message successfully, {send_receipt}")


async def test_transaction_message():
    credentials = SessionCredentials("username", "password")
    credentials_provider = SessionCredentialsProvider(credentials)
    client_config = ClientConfig(
        endpoints=Endpoints("endpoint"),
        session_credentials_provider=credentials_provider,
        ssl_enabled=True,
    )
    topic = Resource()
    topic.name = "transaction_topic"
    msg = ProtoMessage()
    msg.topic.CopyFrom(topic)
    msg.body = b"My Transaction Message Body"
    sysperf = SystemProperties()
    sysperf.message_id = MessageIdCodec.next_message_id()
    msg.system_properties.CopyFrom(sysperf)
    logger.debug(f"{msg}")
    producer = Producer(client_config, topics={"transaction_topic"})
    message = Message(topic.name, msg.body)
    await producer.start()
    # await asyncio.sleep(10)
    transaction = producer.begin_transaction()
    send_receipt = await producer.send(message, transaction)
    logger.info(f"Send message successfully, {send_receipt}")
    await transaction.commit()


async def test_retry_and_isolation():
    credentials = SessionCredentials("username", "password")
    credentials_provider = SessionCredentialsProvider(credentials)
    client_config = ClientConfig(
        endpoints=Endpoints("endpoint"),
        session_credentials_provider=credentials_provider,
        ssl_enabled=True,
    )
    topic = Resource()
    topic.name = "normal_topic"
    msg = ProtoMessage()
    msg.topic.CopyFrom(topic)
    msg.body = b"My Message Body"
    sysperf = SystemProperties()
    sysperf.message_id = MessageIdCodec.next_message_id()
    msg.system_properties.CopyFrom(sysperf)
    logger.info(f"{msg}")
    producer = Producer(client_config, topics={"normal_topic"})
    message = Message(topic.name, msg.body)
    with patch.object(producer.client_manager, 'send_message', new_callable=MagicMock) as mock_send:
        mock_send.side_effect = Exception("Forced Exception for Testing")
        await producer.start()

        try:
            await producer.send(message)
        except Exception:
            logger.info("Exception occurred as expected")

        assert mock_send.call_count == producer.get_retry_policy().get_max_attempts(), "Number of attempts should equal max_attempts."
        logger.debug(producer.isolated)
        assert producer.isolated, "Endpoint should be marked as isolated after an error."

    logger.info("Test completed successfully.")

if __name__ == "__main__":
    asyncio.run(test())
    asyncio.run(test_delay_message())
    asyncio.run(test_fifo_message())
    asyncio.run(test_transaction_message())
    asyncio.run(test_retry_and_isolation())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
from typing import List

from filter_expression import ExpressionType
from google.protobuf.duration_pb2 import Duration
from message import MessageView
from rocketmq.client import Client
from rocketmq.protocol.definition_pb2 import \
    FilterExpression as ProtoFilterExpression
from rocketmq.protocol.definition_pb2 import FilterType
from rocketmq.protocol.definition_pb2 import Resource as ProtoResource
from rocketmq.protocol.service_pb2 import \
    ReceiveMessageRequest as ProtoReceiveMessageRequest


class ReceiveMessageResult:
    def __init__(self, endpoints, messages: List['MessageView']):
        self.endpoints = endpoints
        self.messages = messages


class Consumer(Client):
    CONSUMER_GROUP_REGEX = re.compile(r"^[%a-zA-Z0-9_-]+$")

    def __init__(self, client_config, consumer_group):
        super().__init__(client_config)
        self.consumer_group = consumer_group

    async def receive_message(self, request, mq, await_duration):
        tolerance = self.client_config.request_timeout
        timeout = tolerance + await_duration
        results = await self.client_manager.receive_message(mq.broker.endpoints, request, timeout)

        messages = [MessageView.from_protobuf(message, mq) for message in results]
        return ReceiveMessageResult(mq.broker.endpoints, messages)

    @staticmethod
    def _wrap_filter_expression(filter_expression):
        filter_type = FilterType.TAG
        if filter_expression.type == ExpressionType.Sql92:
            filter_type = FilterType.SQL
        return ProtoFilterExpression(
            type=filter_type,
            expression=filter_expression.expression
        )

    def wrap_receive_message_request(self, batch_size, mq, filter_expression, await_duration, invisible_duration):
        group = ProtoResource()
        group.name = self.consumer_group
        return ProtoReceiveMessageRequest(
            group=group,
            message_queue=mq.to_protobuf(),
            filter_expression=self._wrap_filter_expression(filter_expression),
            long_polling_timeout=Duration(seconds=await_duration),
            batch_size=batch_size,
            auto_renew=False,
            invisible_duration=Duration(seconds=invisible_duration)
        )



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hashlib
import hmac

master_broker_id = 0


def number_to_base(number, base):
    alphabet = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    if number == 0:
        return alphabet[0]

    result = []
    while number:
        number, remainder = divmod(number, base)
        result.append(alphabet[remainder])

    return "".join(reversed(result))


def sign(access_secret: str, datetime: str) -> str:
    digester = hmac.new(
        bytes(access_secret, encoding="UTF-8"),
        bytes(datetime, encoding="UTF-8"),
        hashlib.sha1,
    )
    return digester.hexdigest().upper()


def get_positive_mod(k: int, n: int):
    result = k % n
    return result + n if result < 0 else result



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import operator
import socket
import time
from datetime import timedelta
from enum import Enum
from functools import reduce

import certifi
from grpc import aio, ssl_channel_credentials
from rocketmq.log import logger
from rocketmq.protocol import service_pb2, service_pb2_grpc
from rocketmq.protocol.definition_pb2 import Address as ProtoAddress
from rocketmq.protocol.definition_pb2 import \
    AddressScheme as ProtoAddressScheme
from rocketmq.protocol.definition_pb2 import Endpoints as ProtoEndpoints


class AddressScheme(Enum):
    Unspecified = 0
    Ipv4 = 1
    Ipv6 = 2
    DomainName = 3

    @staticmethod
    def to_protobuf(scheme):
        if scheme == AddressScheme.Ipv4:
            return ProtoAddressScheme.IPV4
        elif scheme == AddressScheme.Ipv6:
            return ProtoAddressScheme.IPV6
        elif scheme == AddressScheme.DomainName:
            return ProtoAddressScheme.DOMAIN_NAME
        else:  # Unspecified or other cases
            return ProtoAddressScheme.ADDRESS_SCHEME_UNSPECIFIED


class Address:
    def __init__(self, host, port):
        self.host = host
        self.port = port

    def to_protobuf(self):
        proto_address = ProtoAddress()
        proto_address.host = self.host
        proto_address.port = self.port
        return proto_address


class Endpoints:
    HttpPrefix = "http://"
    HttpsPrefix = "https://"
    DefaultPort = 80
    EndpointSeparator = ":"

    def __init__(self, endpoints):
        self.Addresses = []

        self.scheme = AddressScheme.Unspecified
        self._hash = None

        if type(endpoints) == str:
            if endpoints.startswith(self.HttpPrefix):
                endpoints = endpoints[len(self.HttpPrefix):]
            if endpoints.startswith(self.HttpsPrefix):
                endpoints = endpoints[len(self.HttpsPrefix):]

            index = endpoints.find(self.EndpointSeparator)
            port = int(endpoints[index + 1:]) if index > 0 else 80
            host = endpoints[:index] if index > 0 else endpoints
            address = Address(host, port)
            self.Addresses.append(address)
            try:
                socket.inet_pton(socket.AF_INET, host)
                self.scheme = AddressScheme.IPv4
            except socket.error:
                try:
                    socket.inet_pton(socket.AF_INET6, host)
                    self.scheme = AddressScheme.IPv6
                except socket.error:
                    self.scheme = AddressScheme.DomainName
            self.Addresses.append(address)

            # Assuming AddressListEqualityComparer exists
            self._hash = 17
            self._hash = (self._hash * 31) + reduce(
                operator.xor, (hash(address) for address in self.Addresses)
            )
            self._hash = (self._hash * 31) + hash(self.scheme)
        else:
            self.Addresses = [
                Address(addr.host, addr.port) for addr in endpoints.addresses
            ]
            if not self.Addresses:
                raise Exception("No available address")

            if endpoints.scheme == "Ipv4":
                self.scheme = AddressScheme.Ipv4
            elif endpoints.scheme == "Ipv6":
                self.scheme = AddressScheme.Ipv6
            else:
                self.scheme = AddressScheme.DomainName
                if len(self.Addresses) > 1:
                    raise Exception(
                        "Multiple addresses are\
                                    not allowed in domain scheme"
                    )

            self._hash = self._calculate_hash()

    def _calculate_hash(self):
        hash_value = 17
        for address in self.Addresses:
            hash_value = (hash_value * 31) + hash(address)
        hash_value = (hash_value * 31) + hash(self.scheme)
        return hash_value

    def __str__(self):
        for address in self.Addresses:
            return str(address.host) + str(address.port)

    def grpc_target(self, sslEnabled):
        for address in self.Addresses:
            return address.host + ":" + str(address.port)
        raise ValueError("No available address")

    def __eq__(self, other):
        if other is None:
            return False
        if self is other:
            return True
        res = self.Addresses == other.Addresses and self.Scheme == other.Scheme
        return res

    def __hash__(self):
        return self._hash

    def to_protobuf(self):
        proto_endpoints = ProtoEndpoints()
        proto_endpoints.scheme = self.scheme.to_protobuf(self.scheme)
        proto_endpoints.addresses.extend([i.to_protobuf() for i in self.Addresses])
        return proto_endpoints


class RpcClient:
    channel_options = [
        ("grpc.max_send_message_length", -1),
        ("grpc.max_receive_message_length", -1),
        ("grpc.connect_timeout_ms", 3000),
    ]

    def __init__(self, endpoints: str, ssl_enabled: bool = True):
        self.__endpoints = endpoints
        self.__cert = certifi.contents().encode(encoding="utf-8")
        if ssl_enabled:
            self.__channel = aio.secure_channel(
                endpoints,
                ssl_channel_credentials(root_certificates=self.__cert),
                options=RpcClient.channel_options,
            )
        else:
            self.__channel = aio.insecure_channel(
                endpoints, options=RpcClient.channel_options
            )
        self.__stub = service_pb2_grpc.MessagingServiceStub(self.__channel)
        self.activity_nano_time = time.monotonic_ns()

    def idle_duration(self):
        return timedelta(
            microseconds=(time.monotonic_ns() - self.activity_nano_time) / 1000
        )

    async def query_route(
        self, request: service_pb2.QueryRouteRequest, metadata, timeout_seconds: int
    ):
        # metadata = [('x-mq-client-id', 'value1')]
        return await self.__stub.QueryRoute(
            request, timeout=timeout_seconds, metadata=metadata
        )

    async def heartbeat(
        self, request: service_pb2.HeartbeatRequest, metadata, timeout_seconds: int
    ):
        return await self.__stub.Heartbeat(
            request, metadata=metadata, timeout=timeout_seconds
        )

    async def send_message(
        self, request: service_pb2.SendMessageRequest, metadata, timeout_seconds: int
    ):
        return await self.__stub.SendMessage(
            request, metadata=metadata, timeout=timeout_seconds
        )

    async def receive_message(
        self, request: service_pb2.ReceiveMessageRequest, metadata, timeout_seconds: int
    ):
        results = self.__stub.ReceiveMessage(
            request, metadata=metadata, timeout=timeout_seconds
        )
        response = []
        try:
            async for result in results:
                if result.HasField("message"):
                    response.append(result.message)
        except Exception as e:
            logger.info("An error occurred: %s", e)
            # Handle error as appropriate for your use case
        return response

    async def query_assignment(
        self,
        request: service_pb2.QueryAssignmentRequest,
        metadata,
        timeout_seconds: int,
    ):
        return await self.__stub.QueryAssignment(
            request, metadata=metadata, timeout=timeout_seconds
        )

    async def ack_message(
        self, request: service_pb2.AckMessageRequest, metadata, timeout_seconds: int
    ):
        return await self.__stub.AckMessage(
            request, metadata=metadata, timeout=timeout_seconds
        )

    async def forward_message_to_dead_letter_queue(
        self,
        request: service_pb2.ForwardMessageToDeadLetterQueueRequest,
        metadata,
        timeout_seconds: int,
    ):
        return await self.__stub.ForwardMessageToDeadLetterQueue(
            request, metadata=metadata, timeout=timeout_seconds
        )

    async def end_transaction(
        self, request: service_pb2.EndTransactionRequest, metadata, timeout_seconds: int
    ):
        return await self.__stub.EndTransaction(
            request, metadata=metadata, timeout=timeout_seconds
        )

    async def notify_client_termination(
        self,
        request: service_pb2.NotifyClientTerminationRequest,
        metadata,
        timeout_seconds: int,
    ):
        return await self.__stub.NotifyClientTermination(
            request, metadata=metadata, timeout=timeout_seconds
        )

    async def change_invisible_duration(
        self,
        request: service_pb2.ChangeInvisibleDurationRequest,
        metadata,
        timeout_seconds: int,
    ):
        return await self.__stub.ChangeInvisibleDuration(
            request, metadata=metadata, timeout=timeout_seconds
        )

    async def send_requests(self, requests, stream):
        for request in requests:
            await stream.send_message(request)

    def telemetry(self, metadata, timeout_seconds: int):
        stream = self.__stub.Telemetry(metadata=metadata, timeout=timeout_seconds)
        return stream


async def test():
    client = RpcClient("rmq-cn-jaj390gga04.cn-hangzhou.rmq.aliyuncs.com:8081")
    request = service_pb2.SendMessageRequest()
    response = await client.send_message(request, 3)
    logger.info(response)


if __name__ == "__main__":
    asyncio.run(test())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import math
from datetime import timedelta

from google.protobuf.duration_pb2 import Duration


class ExponentialBackoffRetryPolicy:
    """A class implementing exponential backoff retry policy."""

    def __init__(self, max_attempts, initial_backoff, max_backoff, backoff_multiplier):
        """Initialize an ExponentialBackoffRetryPolicy instance.

        :param max_attempts: Maximum number of retry attempts.
        :param initial_backoff: Initial delay duration before the first retry.
        :param max_backoff: Maximum delay duration between retries.
        :param backoff_multiplier: Multiplier that determines the delay factor between retries.
        """
        self._max_attempts = max_attempts
        self.initial_backoff = initial_backoff
        self.max_backoff = max_backoff
        self.backoff_multiplier = backoff_multiplier

    def get_max_attempts(self):
        """Get maximum number of retry attempts.

        :return: Maximum number of retry attempts.
        """
        return self._max_attempts

    def inherit_backoff(self, retry_policy):
        """Inherit backoff parameters from another retry policy.

        :param retry_policy: The retry policy to inherit from.
        :return: An instance of ExponentialBackoffRetryPolicy with inherited parameters.
        :raise ValueError: If the strategy of the retry policy is not ExponentialBackoff.
        """
        if retry_policy.strategy_case != "ExponentialBackoff":
            raise ValueError("Strategy must be exponential backoff")
        return self._inherit_backoff(retry_policy.exponential_backoff)

    def _inherit_backoff(self, retry_policy):
        """Inherit backoff parameters from another retry policy.

        :param retry_policy: The retry policy to inherit from.
        :return: An instance of ExponentialBackoffRetryPolicy with inherited parameters.
        """
        return ExponentialBackoffRetryPolicy(self._max_attempts,
                                             retry_policy.initial.ToTimedelta(),
                                             retry_policy.max.ToTimedelta(),
                                             retry_policy.multiplier)

    def get_next_attempt_delay(self, attempt):
        """Calculate the delay before the next retry attempt.

        :param attempt: The number of the current attempt.
        :return: The delay before the next attempt.
        """
        delay_seconds = min(
            self.initial_backoff.total_seconds() * math.pow(self.backoff_multiplier, 1.0 * (attempt - 1)),
            self.max_backoff.total_seconds())
        return timedelta(seconds=delay_seconds) if delay_seconds >= 0 else timedelta(seconds=0)

    @staticmethod
    def immediately_retry_policy(max_attempts):
        """Create a retry policy that makes immediate retries.

        :param max_attempts: Maximum number of retry attempts.
        :return: An instance of ExponentialBackoffRetryPolicy with no delay between retries.
        """
        return ExponentialBackoffRetryPolicy(max_attempts, timedelta(seconds=0), timedelta(seconds=0), 1)

    def to_protobuf(self):
        """Convert the ExponentialBackoffRetryPolicy instance to protobuf.

        :return: A protobuf message that represents the ExponentialBackoffRetryPolicy instance.
        """
        exponential_backoff = {
            'Multiplier': self.backoff_multiplier,
            'Max': Duration.FromTimedelta(self.max_backoff),
            'Initial': Duration.FromTimedelta(self.initial_backoff)
        }
        return {
            'MaxAttempts': self._max_attempts,
            'ExponentialBackoff': exponential_backoff
        }



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os

logger = logging.getLogger("rocketmqlogger")
logger.setLevel(logging.DEBUG)

log_path = os.path.join(
    os.path.expanduser("~"), "logs", "rocketmq", "rocketmq-client.log"
)
file_handler = logging.FileHandler(log_path)
file_handler.setLevel(logging.DEBUG)

console_handler = logging.StreamHandler()
console_handler.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s [%(levelname)s] [%(process)d] [%(threadName)s] [%(filename)s#%(funcName)s:%(lineno)d] %(message)s"
)
file_handler.setFormatter(formatter)
console_handler.setFormatter(formatter)

logger.addHandler(file_handler)
logger.addHandler(console_handler)



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: apache/rocketmq/v2/admin.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x61pache/rocketmq/v2/admin.proto\x12\x12\x61pache.rocketmq.v2\"\x95\x01\n\x15\x43hangeLogLevelRequest\x12>\n\x05level\x18\x01 \x01(\x0e\x32/.apache.rocketmq.v2.ChangeLogLevelRequest.Level\"<\n\x05Level\x12\t\n\x05TRACE\x10\x00\x12\t\n\x05\x44\x45\x42UG\x10\x01\x12\x08\n\x04INFO\x10\x02\x12\x08\n\x04WARN\x10\x03\x12\t\n\x05\x45RROR\x10\x04\"(\n\x16\x43hangeLogLevelResponse\x12\x0e\n\x06remark\x18\x01 \x01(\t2r\n\x05\x41\x64min\x12i\n\x0e\x43hangeLogLevel\x12).apache.rocketmq.v2.ChangeLogLevelRequest\x1a*.apache.rocketmq.v2.ChangeLogLevelResponse\"\x00\x42=\n\x12\x61pache.rocketmq.v2B\x07MQAdminP\x01\xa0\x01\x01\xd8\x01\x01\xf8\x01\x01\xaa\x02\x12\x41pache.Rocketmq.V2b\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'apache.rocketmq.v2.admin_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\022apache.rocketmq.v2B\007MQAdminP\001\240\001\001\330\001\001\370\001\001\252\002\022Apache.Rocketmq.V2'
  _CHANGELOGLEVELREQUEST._serialized_start=55
  _CHANGELOGLEVELREQUEST._serialized_end=204
  _CHANGELOGLEVELREQUEST_LEVEL._serialized_start=144
  _CHANGELOGLEVELREQUEST_LEVEL._serialized_end=204
  _CHANGELOGLEVELRESPONSE._serialized_start=206
  _CHANGELOGLEVELRESPONSE._serialized_end=246
  _ADMIN._serialized_start=248
  _ADMIN._serialized_end=362
# @@protoc_insertion_point(module_scope)



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict

from google.protobuf.duration_pb2 import Duration
from rocketmq.filter_expression import ExpressionType
from rocketmq.log import logger
from rocketmq.protocol.definition_pb2 import \
    FilterExpression as ProtoFilterExpression
from rocketmq.protocol.definition_pb2 import FilterType as ProtoFilterType
from rocketmq.protocol.definition_pb2 import Resource as ProtoResource
from rocketmq.protocol.definition_pb2 import Settings as ProtoSettings
from rocketmq.protocol.definition_pb2 import Subscription as ProtoSubscription
from rocketmq.protocol.definition_pb2 import \
    SubscriptionEntry as ProtoSubscriptionEntry

from .settings import ClientType, ClientTypeHelper, Settings


# Assuming a simple representation of FilterExpression for the purpose of this example
class FilterExpression:
    def __init__(self, type, expression):
        self.Type = type
        self.Expression = expression


class SimpleSubscriptionSettings(Settings):

    def __init__(self, clientId, endpoints, consumerGroup, requestTimeout, longPollingTimeout,
                 subscriptionExpressions: Dict[str, FilterExpression]):
        super().__init__(clientId, ClientType.SimpleConsumer, endpoints, None, requestTimeout)
        self._group = consumerGroup  # Simplified as string for now
        self._longPollingTimeout = longPollingTimeout
        self._subscriptionExpressions = subscriptionExpressions

    def Sync(self, settings: ProtoSettings):
        if not isinstance(settings, ProtoSettings):
            logger.error(f"[Bug] Issued settings doesn't match with the client type, clientId={self.ClientId}, clientType={self.ClientType}")

    def to_protobuf(self):
        subscriptionEntries = []

        for key, value in self._subscriptionExpressions.items():
            topic = ProtoResource()
            topic.name = key

            subscriptionEntry = ProtoSubscriptionEntry()
            filterExpression = ProtoFilterExpression()

            if value.type == ExpressionType.Tag:
                filterExpression.type = ProtoFilterType.TAG
            elif value.type == ExpressionType.Sql92:
                filterExpression.type = ProtoFilterType.SQL
            else:
                logger.warn(f"[Bug] Unrecognized filter type={value.Type} for simple consumer")

            filterExpression.expression = value.expression
            subscriptionEntry.topic.CopyFrom(topic)
            subscriptionEntries.append(subscriptionEntry)

        subscription = ProtoSubscription()
        group = ProtoResource()
        group.name = self._group
        subscription.group.CopyFrom(group)
        subscription.subscriptions.extend(subscriptionEntries)
        duration_longPollingTimeout = Duration(seconds=self._longPollingTimeout)
        subscription.long_polling_timeout.CopyFrom(duration_longPollingTimeout)

        settings = super().to_protobuf()
        settings.access_point.CopyFrom(self.Endpoints.to_protobuf())  # Assuming Endpoints has a to_protobuf method
        settings.client_type = ClientTypeHelper.to_protobuf(self.ClientType)

        settings.request_timeout.CopyFrom(Duration(seconds=int(self.RequestTimeout.total_seconds())))
        settings.subscription.CopyFrom(subscription)

        return settings



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket

from definition import Encoding, EncodingHelper, MessageType, MessageTypeHelper
from google.protobuf.timestamp_pb2 import Timestamp
from message import Message
from message_id_codec import MessageIdCodec
from protocol.definition_pb2 import Message as ProtoMessage
from protocol.definition_pb2 import Resource, SystemProperties
from rocketmq.log import logger


class PublishingMessage(Message):
    def __init__(self, message, publishing_settings, tx_enabled=False):
        self.message = message
        self.publishing_settings = publishing_settings
        self.tx_enabled = tx_enabled
        self.message_type = None

        max_body_size_bytes = publishing_settings.get_max_body_size_bytes()
        if len(message.body) > max_body_size_bytes:
            raise IOError(f"Message body size exceed the threshold, max size={max_body_size_bytes} bytes")

        self.message_id = MessageIdCodec.next_message_id()

        if not message.message_group and not message.delivery_timestamp and not tx_enabled:
            self.message_type = MessageType.NORMAL
            return

        if message.message_group and not tx_enabled:
            self.message_type = MessageType.FIFO
            return

        if message.delivery_timestamp and not tx_enabled:
            self.message_type = MessageType.DELAY
            return

        if message.message_group or message.delivery_timestamp or not tx_enabled:
            pass

        self.message_type = MessageType.TRANSACTION
        logger.debug(self.message_type)

    def to_protobuf(self, queue_id):
        system_properties = SystemProperties(
            keys=self.message.keys,
            message_id=self.message_id,
            # born_timestamp=Timestamp.FromDatetime(dt=datetime.datetime.utcnow()),
            born_host=socket.gethostname(),
            body_encoding=EncodingHelper.to_protobuf(Encoding.IDENTITY),
            queue_id=queue_id,
            message_type=MessageTypeHelper.to_protobuf(self.message_type)
        )
        if self.message.tag:
            system_properties.tag = self.message.tag

        if self.message.delivery_timestamp:
            timestamp = Timestamp()
            timestamp.FromDatetime(self.message.delivery_timestamp)
            system_properties.delivery_timestamp.CopyFrom(timestamp)

        if self.message.message_group:
            system_properties.message_group = self.message.message_group

        topic_resource = Resource(name=self.message.topic)

        return ProtoMessage(
            topic=topic_resource,
            body=self.message.body,
            system_properties=system_properties,
            user_properties=self.message.properties
        )



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import importlib.metadata
import uuid

from rocketmq.client_config import ClientConfig
from rocketmq.utils import sign


class Signature:
    __AUTHORIZATION_KEY = "authorization"

    __DATE_TIME_KEY = "x-mq-date-time"
    __DATE_TIME_FORMAT = "%Y%m%dT%H%M%SZ"

    __SESSION_TOKEN_KEY = "x-mq-session-token"
    __CLIENT_ID_KEY = "x-mq-client-id"
    __REQUEST_ID_KEY = "x-mq-request-id"
    __LANGUAGE_KEY = "x-mq-language"
    __CLIENT_VERSION_KEY = "x-mq-client-version"
    __PROTOCOL_VERSION = "x-mq-protocol"

    __ALGORITHM = "MQv2-HMAC-SHA1"
    __CREDENTIAL = "Credential"
    __SIGNED_HEADERS = "SignedHeaders"
    __SIGNATURE = "Signature"

    @staticmethod
    def sign(client_config: ClientConfig, client_id: str):
        date_time = datetime.datetime.now().strftime(Signature.__DATE_TIME_FORMAT)
        metadata = [
            (Signature.__LANGUAGE_KEY, "PYTHON"),
            (Signature.__PROTOCOL_VERSION, "v2"),
            (Signature.__CLIENT_VERSION_KEY, importlib.metadata.version("rocketmq")),
            (
                Signature.__DATE_TIME_KEY,
                date_time,
            ),
            (Signature.__REQUEST_ID_KEY, str(uuid.uuid4())),
            (Signature.__CLIENT_ID_KEY, client_id),
        ]
        if not client_config.session_credentials_provider:
            return metadata
        session_credentials = (
            client_config.session_credentials_provider.get_credentials()
        )
        if not session_credentials:
            return metadata
        if session_credentials.security_token:
            metadata.append(
                (Signature.__SESSION_TOKEN_KEY, session_credentials.security_token)
            )
        if (not session_credentials.access_key) or (
            not session_credentials.access_secret
        ):
            return metadata
        signature = sign(session_credentials.access_secret, date_time)
        authorization = (
            Signature.__ALGORITHM
            + " "
            + Signature.__CREDENTIAL
            + "="
            + session_credentials.access_key
            + ", "
            + Signature.__SIGNED_HEADERS
            + "="
            + Signature.__DATE_TIME_KEY
            + ", "
            + Signature.__SIGNATURE
            + "="
            + signature
        )
        metadata.append((Signature.__AUTHORIZATION_KEY, authorization))
        return metadata
